from mathutils.Vectorial import *
from mathutils.GeometricObjects import *
from mathutils.TransMatrices import *

class NoIntersection:
	pass


# Dare a questa eccezione un nome decente
class NoLimit:
	pass

# test per la comparazione di coppie.
def __compareFunction(x, y):
	if x[0] < y[0]:
		return -1
	elif x[0] == y[0]:
			return 0
	else:
		return 1


def __compareFunction2(c1, c2):
	l1 = abs((c1[1] - c1[0]).getVector())
	l2 = abs((c2[1] - c2[0]).getVector())
	if l1 < l2:
		return -1
	elif l1 == l2:
		return 0
	else:
		return 1


"""Problema, dati due segmenti, estendere il primo in modo che incida sul secondo.
   1 - Verifico che i segmenti siano coplanari. Se cio' non avviene non e' possibile eseguire l'operazione di
       estensione.
   2 - Calcolo il punto di intersezione tra le rette contenenti i segmenti (pi).
   3 - Verifico che tale punto giaccia sul secondo segmento (il segmento limite). Se cio' non accade l'operazione
       di estensione non puo' essere completata.
   4 - Calcolo l'equazione parametrica del primo segmento. Per il parametro t=0 otterro' un punto estremo
       del segmento, per il parametro t=1, otterro' l'altro estremo.
       Il punto pi, giace sulla retta contenente il segmento da estendere
       (il primo segmento appunto), pertanto ad esso corrisponde un valore del parametro t. Calcolo tale valore.
       Ordino i due punti del segmento da estendere e il punto pi in base al valore del parametro t. I punti
       estremi di tale ordinamento sono gli estremi del segmento ottenuto dopo l'estensione."""





def extendP(pts1, pts2, c):
	"""Estende una polilinea su un'altra polilinea (pts1 su pts2). Se c = 0 estende il segmento iniziale
	   della polilinea, se c != 0, estende il segmento finale."""
	if c == 0:
		# Punti del segmento iniziale della polilinea
		p1 = pts1[1]
		p2 = pts1[0]
	else:
		# Punti del segmento finale della polilinea
		p1 = pts1[len(pts1) - 2]
		p2 = pts1[len(pts1) - 1]

	result = []

	# Per ogni segmento della polilinea limite...
	for i in range(0, len(pts2) - 1):
		try:
			#Verifico se il segmento p1 p2 puo' essere esteso sul segmento pts2[i] pts2[i+1]
			nl = extend(p1, p2, pts2[i], pts2[i+1], 1)
			# Se la chiamata extend va a buon fine, ottengo il segmento esteso e lo aggiungo
			# alla lista di risultati.
			result.append(nl)
		except NoIntersection: # La chiamata extend non e' andata a buon fine.
			# Non faccio nulla visto che non vi e' punto di estensione
			pass
		except NoLimit:    # La chiamata extend non e' andata a buon fine
			# Non faccio nulla visto che non vi e' punto di estensione
			pass
	result.sort(__compareFunction2) # Ordino i segmenti in base alla loro lunghezza
	return result








def extend(pa1, pb1, pa2, pb2, c):
	""" Estende il segmento pa1-pb1 in modo che incida sul segmento pa2-pb2 . c specifica in che verso
	    il segmento pa1-pb1 deve essere esteso. Se c==0 allora il segmento viene esteso nel verso
	    pb1->pa1, se c != 0 allora il segmento viene esteso nel verso pa1->pb1."""
	n = ((pb1 - pa1).getVector()**(pb2 - pa2).getVector()).normalize()

	# Il test seguente garantisce la non intersezione tra le rette contenenti i due segmenti.
	if abs(n*(pa2 - pa1).getVector().normalize()) > Global.zero:  # Vedere documentazione
	#Allora le due rette non possono intersecarsi
		raise NoIntersection

	try:    # pa1 pb1 e pa2 potrebbero essere collineari.
		plane = Plane(pa1, pb1, pa2)
		# Piano contenente pa1, pb1, pa2 e pb2 (infatti il test precedente ha
	        # gia' dimostrato che i due segmenti giacciono sul medesimo piano e quindi
		# anche i punti che li compongono)
	except PointsCollinear:
		plane = Plane(pa1, pb1, pb2)

	# L'oggetto Plane cosi' ricavato, mi permette di ricavare i tre vettori necessari a modificare il
	# sistema di riferimento, facendo coincidere il nuovo piano xy con plane.

	ms = get_system_reference_matrix(pa1, plane.get_x_vector(), plane.get_y_vector(), plane.get_z_vector())

	# Trasformo i punti dei segmenti in modo che giacciano sul piano xy.
	pa = pa1*ms
	pb = pb1*ms
	pc = pa2*ms
	pd = pb2*ms

	# Calcolo il punto di intersezione tra le rette contenenti i due segmenti.
	pi = intersection_xy(pa, pb-pa, pc, pd-pc)

	s = segment(pc, pd)
	t = s.test(pi)
	# pi, deve giacere sul segmento pc-pd quindi...
	if t >= 0.0 and t <= 1.0:
		if c == 0:
			aus = pa
			pa = pb
			pb = aus
		s = Segment(pa, pb)

		if s.test(pi) > 1.0:
			return [pa*ms.inv(), pi*ms.inv()]
		else:
			return [pa1, pb1]
	else:
		# Se pi non giace sul segmento pc-pd, significa che il segmento pc-pd non e' un segmento limite
		# corretto, pertanto genero una eccezione
		raise NoLimit



def extend(pa1, pb1, pa2, pb2):
	""" Estende il segmento pa1-pb1 in modo che incida sul segmento pa2-pb2 ."""
	result = []

	v1 = (pb1 - pa1).getVector()
	v2 = (pb2 - pa2).getVector()
	v3 = (pa2 - pa1).getVector()

	n =  v1**v2

	# Il test seguente garantisce la non intersezione tra le rette contenenti i due segmenti.
	if abs(n.normalize()*v3.normalize()) > Global.zero:  # Vedere documentazione
	#Allora le due rette non possono intersecarsi
		raise NoIntersection

	try:    # pa1 pb1 e pa2 potrebbero essere collineari.
		plane = Plane(pa1, pb1, pa2)
		# Piano contenente pa1, pb1, pa2 e pb2 (infatti il test precedente ha
	        # gia' dimostrato che i due segmenti giacciono sul medesimo piano e quindi
		# anche i punti che li compongono)
	except PointsCollinear:
		plane = Plane(pa1, pb1, pb2)

	# L'oggetto Plane cosi' ricavato, mi permette di ricavare i tre vettori necessari a modificare il
	# sistema di riferimento, facendo coincidere il nuovo piano xy con plane.

	ms = get_system_reference_matrix(pa1, plane.get_x_vector(), plane.get_y_vector(), plane.get_z_vector())

	# Trasformo i punti dei segmenti in modo che giacciano sul piano xy.
	pa = pa1*ms
	pb = pb1*ms
	pc = pa2*ms
	pd = pb2*ms

	# Calcolo il punto di intersezione tra le rette contenenti i due segmenti.
	pi = intersection_xy(pa, pb-pa, pc, pd-pc)

	s = Segment(pc, pd)
	t = s.test(pi)
	# pi, deve giacere sul segmento pc-pd quindi...
	if t >= 0.0 and t <= 1.0:
		s = Segment(pa, pb)

		# pi sara uno degli estremi del nuovo segmento dopo l'estensione
		result.append((0.0, pa)) # 0.0 e' il valore del parametro t che mi permette di ottenere pa
		result.append((1.0, pb)) # 1.0 e' il valore del parametro t che mi permette di ottenere pb
		result.append((s.test(pi), pi)) #s.test(pi) e' il valore del parametro t che mi permette di ottenere pi
		result.sort(__compareFunction) # In base al valore di t ordino pa, pb, pi
		# Dopo l'ordinamento della lista result, gli estremi della stessa saranno i punti del segmento cercato

		# Rispristino il vecchio sistema di coordinate attraverso la matrice inversa di ms, e produco i
		# due punti del nuovo segmento (dopo l'estensione)
		return [result[0][1]*ms.inv(), result[2][1]*ms.inv()]
	else:
		# Se pi non giace sul segmento pc-pd, significa che il segmento pc-pd non e' un segmento limite
		# corretto, pertanto genero una eccezione
		raise NoLimit


