"""
	##########################
	Modulo scritto da
		Gianmario Salvetti
	##########################

Questo modulo contiene definizioni di classi rappresentanti oggetti geometrici.
Vedere la documentazione per comprendere il significato geometrico delle operazioni
associate ai vari oggetti.

"""
#Attenzione convertire i valori immessi a float o double


#Attenzione, queste classi vogliono essere il piu' generali possibile. Ad esempio si vuole permettere la somma
#tra punti e vettori oltre che solo tra punti. Questo modo di procedre puo' risultare comodo, ma nello stesso
#tempo nasconde delle insidie. Ad esempio se sommo un punto ad un vettore, ottengo un vettore, ma se sommo
#un vettore ad un punto ottengo un nuovo punto. Questo comportamento asimmetrico e' molto pericoloso. In linea
#di principio sarebbe meglio eseguire operazioni tra oggetti dello stesso tipo. Quando cio' non e' possibile
#o non e' voluto, prestate molta  attenzione.!!!!!


from Vectorial import *   #Ottimizzare
from Global import *

class PointsCollinear:
	pass


class ExternPoint:
	pass


class Point:
	"""Punto tridimensionale nello spazio"""

	def __init__(self, a=None, b=None, c=None):
		if a == None: # Non e' stato passato nessun parametro, quindi imposto il punto a (0,0,0)
			self.__constructor(0.0, 0.0, 0.0)
		elif b == None: # assumo che a sia un vettore
			print "\nSconsigliato!!!! Usare la funzione toPoint() \n"
			self.__constructor(a[0], a[1], a[2])   #Bruttissimo, da eliminare (usare il toPoint)
		elif c == None: # Sono state passate solo due coordinate, imposto z=0.0
			self.__constructor(a, b, 0.0)
		else:
			self.__constructor(a,b,c) # Imposto le 3 coordinate
	def __constructor(self, x, y, z):
		self.__data = Vector(4)
		self.__data[0] = x
		self.__data[1] = y
		self.__data[2] = z
		self.__data[3] = 1.0

	# Le seguenti tre funzioni sono ridondanti. In effetti, sarebbero sufficienti i metodi __getitem__
	# e __setitem__. L'utilizzo di questi tre metodi puo' comunque aumentare la leggibilita' del codice.
	def getX(self):
		"""Restituisce la coordinata x del punto self"""
		return self.__data[0] / self.__data[3]
	def getY(self):
		"""Restituisce la coordinata y del punto self"""
		return self.__data[1] / self.__data[3]
	def getZ(self):
		"""Restituisce la coordinata z del punto self"""
		return self.__data[2] / self.__data[3]
	def setX(self, value):
		"""Imposta la coordinata x del punto self"""
		self.__data[0] = self.__data[3] * value
	def setY(self, value):
		"""Imposta la coordinata y del punto self"""
		self.__data[1] = self.__data[3] * value
	def setZ(self, value):
		"""Imposta la coordinata z del punto self"""
		self.__data[2] = self.__data[3] * value
	def __getitem__(self, index):
		return self.__data[index] / self.__data[3]
	def __setitem__(self, index, value):
		self.__data[index] = value*self.__data[3]
	def __len__(self):
		return 3
	def __sub__(self, p):
		"""Resituisce un punto che ha per coordinate la differenza tra le corrispondenti coordinate del punto
		    self e del punto p"""
		result = Point(self.getX() - p[0], self.getY() - p[1], self.getZ() - p[2])
		return result
	def __add__(self, p):
		"""Resituisce un punto che ha per coordinate la somma tra le corrispondenti coordinate del punto
		    self e del punto p (p puo' anche essere un vettore di dimensione 3)"""
		return Point(self.getX() + p[0], self.getY() + p[1], self.getZ() + p[2])
	def __div__(self, n):
		"""Restituisce un punto che ha per coordinate, le coordinate del punto self, divise per n"""
		return Point(self.getX() / n, self.getY() / n, self.getZ() / n)
	def __mul__(self, m):
		"""Produce  un punto  che e'  la trasformazione di self  attraverso la matrice m. """
		result = Point(0.0, 0.0, 0.0)
		# Il vettore di dati di self, viene moltiplicato per m
		v = self.__data * m
		result.__data = v
		return result
	def getVector(self):
		""""Restituisce un vettore di dimensione 3, contenente le coordinate del punto self"""
		result = Vector(3)
		result[0] = self.getX()
		result[1] = self.getY()
		result[2] = self.getZ()
		return result
	def copy(self, p):
		"""Copia le coordinate del punto p in self"""
		self.setX(p[0])
		self.setY(p[1])
		self.setZ(p[2])
	def clone(self):
		return Point(self.getX(), self.getY(), self.getZ())
	def __repr__(self):
		return "(%f, %f, %f)" % (self.getX(), self.getY(), self.getZ())

def toPoint(v):
	# Trasforma una lista di valori in un oggetto di tipo Point
	l = len(v)
	if l == 3:
		return Point(v[0], v[1], v[2])
	elif l == 2:
		return Point(v[0], v[1])
	elif l == 1:
		return Point(v[0], 0.0)
	else:
		return None

class Points:
	def __init__(self, n):
		"""Crea n punti nulli"""
		self.__data = Matrix(n, 4)
		for i in range(0, n):
			self.__data[i][3] = 1.0
	def clone(self):
		result = Points(len(self))
		for i in range(0, len(self)):
			result[i] = self[i]
		return result
	def __mul__(self, m):
		"""Trasforma i punti self utilizzando la matrice m"""
		if m.row() == m.column() and m.row() == 4:
			result = Points(0)
			result.__data = self.__data * m
			return result
		else:
			raise ErrorDimMatrix
	def __getitem__(self, index):
		""" Produce il punto nella posizione index """
		result = Point(0.0, 0.0, 0.0)
		result._Point__data = self.__data.getRow(index)
		return result
	def __setitem__(self, i,  p):
		"""self[i] non e' un reference a p. E' una copia di p."""
		p1 = self[i]
		p1.setX(p.getX())
		p1.setY(p.getY())
		p1.setZ(p.getZ())
	def __len__(self):
		return self.__data.row()
	def addPoint(self, l=[]):
		self.__data.addRow(l)
		self.__data[len(self)-1][3] = 1.0 #Imposto la coordinata omogenea
	def __repr__(self):
		result = ""
		for i in range(0, len(self)):
			result = result + "%d: %s\n" % (i, Point.__repr__(self[i]))
		return result


class Plane:
	def __init__(self, p1, p2, p3):
		"""Classe per la creazione di oggetti di tipo piano. Permette di ottenere:\n
		  	1 - Normale al piano \n
			2 - Un punto giacente sul piano \n
			3 - Due vettori giagenti sul piano \n
			4 - Coefficiente dell'equazione del piano."""
		self.__x = (p2 - p1).getVector().normalize()
		self.__n = (( self.__x )**( (p3 - p2).getVector() )).normalize()
		if abs(self.__n) <= Global.zero:
			raise PointsCollinear
		self.__y = (self.__n**self.__x).normalize()
		self.__d = -(p1.getVector() * self.__n)
		self.__p = p1.clone()
	def get_point(self):
		"""Produce un punto appartenenente al piano"""
		return self.__p
	def get_n_vector(self):
		"""Produce la normale al piano. Le componenti della normale sono i coefficienti delle incognite
		    dell'equazione che rappresenta questo piano."""
		return self.__n
	def get_x_vector(self):
		"""Produce un vettore x  che giace sul piano"""
		return self.__x
	def get_y_vector(self):
		"""Produce un vettore y ortogonale ad x che giace sul piano"""
		return self.__y
	def get_z_vector(self):
		"""Produce un vettore z, ortogonale ad x e ad y (e' un altro modo di chiamare la
		     normale del piano)"""
		return self.__n
	def get_d_value(self):
		"""Produce il valore del termine noto dell'equazione rappresentante il piano"""
		return self.__d
	def test(self, p):
		"""Questo metodo testa l'appartenenza al piano del punto p"""
		if abs(self.__n*p.getVector()) > zero :
			return False
		else:
			return True

class Segment:
	def __init__(self, pa, pb):
		self.pa = pa.clone() #Ottimizzare
		self.pb = pb.clone() #Ottimizzare
	def __init__(self, pa, pb):
		self.pa = pa
		self.pb = pb
	def getMid(self):
		return (self.pa + self.pb)/2.0
	def getVector(self):
		"""Produce il vettore direzione del segmento (da pa a pb) """
		return (self.pb - self.pa).getVector()
	def getPoint(self, t=0.0):
		"""Produce un punto del segmento. Se il valore di t non e' compreso tra zero e uno produce un
		   punto appartenenete alla retta contenente il segmento."""
		return self.pa + t*self.getVector() #Ottimizzare
	def test(self, p):
		"""Se il punto p non appartiene alla linea contenente il segmento self, genera una eccezione,
		   altrimenti, produce un valore reale. Se tale valore risulta essere compreso tra zero e uno,
		   allora p appartiene al segmento self, altrimenti p appartiene alla retta contenente il
		   segmento self. Il valore del parametro t, puo' risultare utile per ordinare dei punti noti
		   di un segmento."""
		# Ottimizzare il test
		v1 = (self.pa - p).getVector()   # Vettore di direzione self.pa->p
		v2 = (self.pb - p).getVector()   # Vettore di direzione self.pb->p
		d = abs((self.pb - self.pa).getVector()) # Lunghezza del segmento pa-pb

		# Ottimizzare il test. Attenzione!!! Non ottimizzare eliminando semplicemente la normalizzazione
		# Vedere il perche' nella documentazione.
		if abs(v1.normalize()**v2.normalize()) <= Global.zero:
			# Se il prodotto scalare di v1 e v2 e' il vettore nullo, significa che v1 v2 sono paralleli,
			# pertanto p giace sulla retta contenente p1-p2.
			absv1 = abs(v1) # lunghezza di v1, ossia del segmento self.pa-p
			absv2 = abs(v2) # lunghezza di v2, ossia del segmento self.pb-p

			if (abs(absv1 + absv2 - d) < Global.zero): #Il punto e' nel segmento
				return absv1 / d
			elif absv1 > absv2:
				return absv1 / d # Questo rapporto esprime il valore del parametro t dell'equazione
				                 # parametrica della retta contenente pa-pb
 			else:   # ...altrimenti significa che il punto pa si trova
				# tra il punto p ed il punto pb.
				return -absv1 / d # Questo rapporto esprime il valore del parametro t dell'equazione
				                  # parametrica della retta contenente pa-pb
		else: # ...altrimenti il punto p non appartiene ne al segmento pa-pb, ne alla retta contenente tale
		      # segmento
			raise ExternPoint
	def __mul__(self, m):
		"""Trasforma il segmento self attraverso la matrice m."""
		return Segment(self.pa*m, self.pb*m)
	def clone(self):
		return Segment(self.pa, self.pb)
	def divide(self, n):
		result = Points(n+1)
		step = 1.0 / n
		for i in range(0, n+1):
			result[i] = self.getPoint(i*step)
		return result
	def __repr__(self):
		return Point.__repr__(self.pa) + " -> " + Point.__repr__(self.pb)

def intersection_xy(p1, v1, p2, v2):
	"""Calcola l'intersezione di due rette nel piano xy. p1 v1 sono rispettivamente il punto
	   ed il vettore direzione della retta 1, p2 v2 il punto e il vettore direzione della retta 2.
	   La coordinata z dei punti p1 e p2 e' ignorata. Vedere la documentazione per l'algoritmo.
	 """

	#m = toMatrix([[+v1[1],+v2[1],0,0], [-v1[0],-v2[0],0,0], [0,0,1,0], [0,0,0,1]])

 	m = Matrix(4,4)
	m[0][0] = +v1[1]
	m[0][1] = +v2[1]
	m[1][0] = -v1[0]
	m[1][1] = -v2[0]
	m[2][2] = 1.0
	m[3][3] = 1.0

	# Per rendere piu' generale possibile questa funzione si e' deciso di non utilizzare i metodi getX(), getY() e
	# getZ() della classe Point.
	b = Point(v1[1]*p1[0] - v1[0]*p1[1], v2[1]*p2[0] - v2[0]*p2[1])
	return b*m.inv()
