"""
	##########################
	Modulo scritto da
		Gianmario Salvetti
	##########################
Vettori e matrici. Questa classe definisce vettori matrice e operazioni correllate:
Operazioni su vettori:\n
	Somma\n
	Sottrazione\n
	Prodotto scalare\n
	Prodotto vettoriale (solo per vettori di dimensione 3)\n
	Scalare per vettore\n
	Vettore diviso scalare\n
	Norma euclidea di un vettore\n
	Normalizzazione di un vettore\n
	Clonazione                   \n
Operazioni su matrici:\n
	Prodotto matrice-matrice\n
	Matrice inversa   # Aggiunto\n
	Scambio tra righe # Aggiunto\n
	Clonazione        # Aggiunto\n
Operazioni su vettori e matrici:\n
	Prodotto vettore-matrice"""

from array import array
from math import sqrt
import Global

class SingularMatrix:
	pass

class ErrorDimMatrix:
	pass

class ErrorDimVector:
	pass

class Vector:
	"""Questa classe implementa un vettore in senso matematico."""
	def __init__(self, n = 3):
		self.__data = []
		selfd = self.__data
		for i in range(0, n):
			selfd.append(0.0)
	def __add__(self, v):
		"""Somma tra loro i vettori self e v"""
		if len(self) == len(v):
			result = Vector(len(v))
			resultd = result.__data
			selfd = self.__data
			vd = v.__data
			for i in range(0, len(self)):
				resultd[i] = selfd[i] + vd[i]
			return result
		else:
			raise ErrorDimVector
	def __sub__(self, v):
		"""Calcola il vettore differenza fra self e v"""
		if len(self) == len(v):
			result = Vector(len(v))
			resultd = result.__data
			selfd = self.__data
			vd = v.__data
			for i in range(0, len(self)):
				resultd[i] = selfd[i] - vd[i]
			return result
		else:
			raise ErrorDimVector
	def __rmul__(self, v):
		"""Calcola il prodotto scalare tra i vettori self e v oppure il prodotto scalare-vettore
		     tra lo scalare self e il vettore v"""
		if self.__class__ == v.__class__: # Lo so che e' brutto ma...
			if len(self) == len(v):
				result = 0
				selfd = self.__data
				vd = v.__data
				for i in range(0, len(self)):
					result += selfd[i] * vd[i]
				return result
			else:
				raise ErrorDimVector
		else:
			result = Vector(len(self))
			resultd = result.__data
			selfd = self.__data
			for i in range(0, len(self)):
				resultd[i] = selfd[i] * v
			return result
	def __mul__(self, m):		
		if self.__class__ == m.__class__: #Prodotto scalare
			if len(self) == len(m):
				result = 0
				selfd = self.__data
				md = m.__data
				for i in range(0, len(self)):
					result += selfd[i] * md[i]
				return result	
			else:
				raise ErrorDimVector
		else:				#Prodotto vettore matrice
			if len(self) == m.row():
				result = Vector(m.row())
				resultd = result.__data
				selfd = self.__data
				md = m._Matrix__data
				for i in range(0, m.row()):
					for k in range(0, m.column()):
						resultd[i] += selfd[k]*md[k][i]
				return result
	def __div__(self, k):
		""" Divide il vettore self per uno scalare k"""
		result = Vector(len(self))
		resultd = result.__data
		selfd = self.__data
		for i in range(0, len(self)):
			resultd[i] = selfd[i] / float(k)
		return result
	def __pow__(self, v):
		"""Calcola il prodotto vettoriale tra self e v. Se i due vettori rappresentano due direzioni, il loro
		     prodotto vettoriale, rappresenta un vettore perpendicolare ad entrambi """
		if len(self) == len(v) and len(self) == 3:
			result = Vector(3)
			resultd = result.__data
			selfd  = self.__data
			vd = v.__data
			for i in range(0, len(self)):
				resultd[i] += selfd[(i+1)%3] * vd[(i+2)%3] - selfd[(i+2)%3] * vd[(i+1)%3]
			return result
		else:
			raise ErrorDimVector
	def __getitem__(self, i):
		return self.__data[i]
	def __setitem__(self, i, value):
		self.__data[i] = value
	def __abs__(self):
		"""Calcola la norma euclidea del vettore self. La norma euclidea fornisce la lunghezza del vettore"""
		return sqrt(self*self)
	def __len__(self):
		return len(self.__data)
	def normalize(self):
		"""Normalizza il vettore self. Un vettore si dice normalizzato quando la sua norma euclidea e' unitaria"""
		q = abs(self)
		if q > Global.zero:
			return self / q
		else:
			return 0.0*self
	def clone(self):
		result = Vector(len(self))
		for i in range(0, len(self)):
			result[i] = self[i]
		return result
	def __repr__(self):
		return list.__repr__(self.__data)

def toVector(v):
	n = len(v)
	result = Vector(n)
	for i in range(0, n):
		result[i] = v[i]
	return result

class Matrix(list):
	"""Questa classe definisce un tipo matrice"""
	def __init__(self, row=3, column=3):
		self.__data = []
		self.__row = row
		self.__column = column
		"""for i in range(0, row):
			self.append(Vector(column))"""

		selfd = self.__data
		for i in range(0, row):
			selfd.append([])
			for j in range(0, column):
				selfd[i].append(0.0)
	def row(self):
		""" Numero di righe della matrice"""
		return self.__row
	def column(self):
		""" Numero di colonne della matrice"""
		return self.__column
	def transpose(self):
		result = Matrix(self.column(), self.row())
		for i in range(0, self.column()):
			for j in range(0, self.row()):
				result[j][i] = self[i][j]
		return result
	def __len__(self):
		return self.__row
	def __mul__(self, m):
		"""Calcola il prodotto tra due matrici."""
		if self.__column == m.__row:
			result = Matrix(self.__row, m.__column)
			resultd = result.__data
			selfd = self.__data
			md = m.__data
			for i in range(0, self.__row):
				for j in range(0, m.__column):
					for k in range(0, self.__column):
						resultd[i][j] += selfd[i][k] * md[k][j] #Ottimizzare
			return result
		else:
			raise ErrorDimMatrix
	def __rmul__(self, v):
		"""Calcola il prodotto tra il vettore v e la matrice self."""
		if len(v) == self.__row:
			result = Vector(len(v))
			resultd = result.__data
			vd = v._Vector__data
			selfd = self.__data
			for i in range(0, self.__column):
				for k in range(0, len(v)):
					resultd[i] += vd[k] * selfd[k][i]
			return result
		else:
			raise ErrorDimMatrix
	def __repr__(self):
		result = "\n"
		for i in range(0, self.__row):
			for j in range(0, self.__column):
				result += `self[i][j]` + ' '
			result += '\n'
		return result
	def addRow(self, l=[]):
		"""Aggiunge una riga alla matrice. Se viene passata una lista vuota, viene aggiunta una riga di zeri.
		    Se viene passata una lista non vuota, vengono inseriti gli elementi della lista e nel caso in cui non
		    siano sufficienti per crerare una riga completa, si aggiungono elementi nulli.  """
		self.__data.append(Vector(self.__column))
		self.__row += 1
		if l != []:
			selfd = self.__data
			for i in range(0, self.__column):
				try:
					selfd[self.__row-1][i] = l[i]
				except IndexError:
					selfd[self.__row-1][i] = 0.0
	def getRow(self, indexRow):
		"""Produce un reference alla riga indexRow, sotto forma di oggetto Vector."""
		result = Vector(self.__column)
		result._Vector__data = self.__data[indexRow]
		return result
	def clone(self):
		"""Crea una copia della matrice self"""

		result = Matrix(self.row(), self.column())
		for i in range(0, self.row()):
			for j in range(0, self.column()):
				result[i][j] = self[i][j]
		return result
	def switchRows(self, i, j):
		"""Scambia tra loro le righe i e j della matrice"""
		aus = self.__data[i]
		self.__data[i] = self.__data[j]
		self.__data[j] = aus
	def __getitem__(self, index):
		return self.__data[index]
	def __pivoting(self, k):
		for i in range(k+1, self.row()):
			if abs(self[k][i]) > Global.zero:
				return i
		return 0
	def inv(self):
		"""Se la matrice self e' quadrata ed invertibile, produce la sua inversa"""
		if self.row() == self.column():
			m = self.clone()
			result = Matrix(self.row(), self.column())
			for i in range(0, self.row()):
				result[i][i] = 1.0

			# Fase 1: Trasformo la matrice m in una matrice triangolare superiore
			# In questa fase puo' essere necessaria la pivotazione
			for k in range(0, m.row()-1):
				# E' necessaria la pivotazione?
				if abs(m[k][k]) <= Global.zero:
					index = self.__pivoting(k)  #Si!
					if index == 0:
						# Se index e' uguale a zero, significa che non e' possibile eseguire
						# la pivotazione nonostante ce ne sia la necessita'. In questo caso
						# la matrice risulta non essere invertibile.
						raise SingularMatrix  # La matrice non e' invertibile
					else:
						m.switchRows(k, index)
						result.switchRows(k, index)

				for i in range(k+1, m.row()):
					f = float(m[i][k]) / float(m[k][k])
					for j in range(0, m.row()):
						m[i][j] = m[i][j] - m[k][j]*f
						result[i][j] = result[i][j] - result[k][j]*f

			# Fase 2: Utilizzo un metodo, per trasformare una matrice in una matrice
			# triangolare inferiore. Poiche' la matrice di partenza e' triangolare superiore,
			# questo metodo trasforma m in una matrice diagonale. In questa fase non serve la
			# pivotazione.
			s1 = range(1, m.row())
			s1.reverse()
			for k in s1:
				s2 = range(0, k)
				s2.reverse()
				for i in s2:
					f = float(m[i][k]) / float(m[k][k])
					s3 = range(0, m.row())
					s3.reverse()
					for j in s3:
						m[i][j] = m[i][j] - m[k][j]*f
						result[i][j] = result[i][j] - result[k][j]*f

			#Fase 3, divido ciascuna riga i-esima della matrice result per il valore m[i][i]
			for i in range(0, m.row()):
				for j in range(0, m.column()):
					result[i][j] = result[i][j] / float(m[i][i])
			return result #Ecco a voi l'inversa di self
		else: #La matrice self, non e' quadrata.
			raise ErrorDimMatrix

def toMatrix(m):
	r = len(m)
	c = len(m[0])
	result = Matrix(r, c)
	for i in range(0, r):
		for j in range(0, c):
			result[i][j] = m[i][j]
	return result

