from mathutils.Vectorial import *
from mathutils.GeometricObjects import *
from mathutils.TransMatrices import *

class NoIntersection:
	pass


# Per il momento produce il punto di intersezione tra le rette che contengono i segmenti.
# Verra' modificata in seguito per produrre il punto esatto.
def inter(pa1, pb1, pa2, pb2):
	""" Calcola il punto di intersezione tra il segmento pa1-pb1 e il segmento pa2-pb2. Se i due
	    segmenti non si intersecano solleva l'eccezione NoIntersection."""
	n = ((pb1 - pa1).getVector()**(pb2 - pa2).getVector()).normalize()
	p1 = pa1.getVector()     # Punto appartenente al segmento 1
	p2 = pa2.getVector()     # Punto appartenente al segmento 2

	# Il test seguente garantisce la non intersezione tra le rette contenenti i due segmenti.
	if abs(n*(p2 - p1).normalize()) > Global.zero:  # Vedere documentazione
	#Allora le due rette non possono intersecarsi
		raise NoIntersection

	
	try: #pa1 pb1 e pa2 potrebbero essere collineari
		plane = Plane(pa1, pb1, pa2)  # Piano contenente pa1, pb1, pa2 e pb2 (infatti il test precedente ha
	                              	      # gia' dimostrato che i due segmenti giacciono sul medesimo piano e quindi
				              # anche i punti che li compongono)
	except PointsCollinear:
		plane = Plane(pa1, pb1, pb2)

	# L'oggetto Plane cosi' ricavato, mi permette di ricavare i tre vettori necessari a modificare il
	# sistema di riferimento, facendo coincidere il nuovo piano xy con plane.

	ms = get_system_reference_matrix(pa1, plane.get_x_vector(), plane.get_y_vector(), plane.get_z_vector())

	pa = pa1*ms
	pb = pb1*ms
	pc = pa2*ms
	pd = pb2*ms

	pi = intersection_xy(pa, pb-pa, pc, pd-pc)

	# pi e' il punto di intersezione tra le rette che contengono i due segmenti. A questo punto e' necessario
	# verificare l'appartenenza del punto pi ad entrambi i segmenti. Se cio' non accade i due segmenti non si
	# intersecano, al contrario delle loro proiezioni.
	
	s = Segment(pa, pb)
	t = s.test(pi)
	print "t:", t
	if t >= 0.0 and t <= 1.0 : #Testa se pi appartiene al segmento s (pa->pb)
		s =	Segment(pc, pd)
		t = s.test(pi)
		print "t:", t
		if t > 1.0 or t < 0.0:   #Testa se pi non appartiene al segmento s (ora pc->pd)
			raise NoIntersection
	else:
		raise NoIntersection
	
	# Rispristino il vecchio sistema di coordinate attraverso la matrice inversa di ms.
	return pi*ms.inv()


