#####################################
# Gianmario Salvetti
#	file Mid.py
# Modulo per lo snap ad oggetto mid.
#####################################

""" 
Osservazioni.
Operando su un oggetto mesh con 300 vertici, la funzione puo' apparire lenta. In effetti, alcune ottimizzazioni sul codice
possono essere fatte. Non bisogna pero' affidarsi alla sola efficienza di questa funzione. Bisogna puntare soprattutto 
sull'ottimizzazione delle strutture dati. In sostanza, quando viene chiamato il metodo getSegments(), non devono essere prodotti
tutti i segmenti costituenti l'oggetto, ma dovrebbero essere prodotti solo una parte di essi (quelli contenuti in un intorno del
cursore-per esempio). Questo limiterebbe il numero di oggetti sui quali la funzione mid deve operare.

"""

# Essere piu' selettivi sui moduli, in modo da rendere piu' leggibile il codice (future ottimizzazioni)
from GeometricObjects import *
from Vectorial import *
from TransMatrices import *

#Potendo disporre anche di una matrice di zoom, sarebbe possibile non dover necessariamente processare tutti i punti.
#Lo zoom, permetterebbe di stabilire un valore di intorno. Il primo punto contenuto nell'intorno specificato verrebbe
#prodotto dalla funzione mid. In questo modo, il numero medio di confronti per trovare il punto desiderato, si ridurrebbe
#a n/2, dove n e' il numero di punti candidati.


#E' possibile ottimizzare il tutto, inserendo i punti medi in una struttura dati piu' consona (una struttura, che elimini i 
#punti doppi).
def __getMiddlePoints(segments):
	#Data una lista di segmenti, crea un oggetto che memorizza i punti medi di ciascun segmento
	result = Points(len(segments))
	for i in range(0, len(segments)):
		#Calcolo il punto medio del segmento segments[i]
		p1 = segments[i][0]
		p2 = segments[i][1]		
		result[i] = ( p1 + p2) / 2.0
	return result

def mid(cursorPosition, object):
	"""Snap mid-point, per il momento funziona solo su segmenti. cursorPosition, e' un oggetto di tipo
	   Point rappresentante la posizione globale del cursore. object e' un oggetto che deve fornire il
	   metodo getSegments().""" 
	# Mi riferisco ai segmenti di object con la variabile segments
	segments = object.getSegments() #I punti di segments potrebbero essere in coordinate locali
	# Calcolo i punti medi di ciascun segmento
	midPoints_f = __getMiddlePoints(segments) #Non sovrascrivere la variabile midPoints_f perche' serve alla fine
	# Trasformo i punti nel sistema di riferimento di visualizzazione ed annullo la coordinata z 
	midPoints = midPoints_f*object.getMatrix()*object.getViewMatrix()*get_XY_projection_matrix() #Ottimizzare
	cursorPosition = cursorPosition*object.getViewMatrix()*get_XY_projection_matrix() #Ottimizzare
	
	# Inizialmente assumo che il punto 0 sia il piu vicino al punto cursorPosition
	result = 0
	# Calcolo la distanza tra il punto 0 e la posizione del cursore
	distance = abs( (midPoints[0] - cursorPosition).getVector() ) #Ottimizzare

	for i in range(1, len(midPoints)):
		# Calcolo la distanza tra il punto i e la posizione del cursore
		idistance = abs((cursorPosition - midPoints[i]).getVector()) # Ottimizzare
		if idistance < distance:
			# Se il punto i e' piu' vicino al cursore rispetto a result...
			distance  = idistance
			# ...il nuovo result diventa i
			result = i
	# midPoints_f, memorizza i punti medi in un sistema di riferimento locale. Debbo quindi trasformare
	# il punto medio risultante in modo da portarlo nel sistema di riferimento globale.
	return midPoints_f[result]*object.getMatrix() #Devo trasformarlo in coordinate globali